//------------------------------------------------------------------
//
//------------------------------------------------------------------
#property indicator_separate_window
#property indicator_buffers 1
#property indicator_color1  Orange
#property indicator_width1  2
#property indicator_level1  0
#property indicator_levelcolor Orange

extern int    Mom_Period  = 4;
extern int    T3_Period   = 28;
extern bool   T3_original = false;
extern double b           = 0.618;
extern double levelUp     =  0.03;
extern double levelDn     = -0.03;
extern int    Price       = PRICE_CLOSE;
extern bool   verticalLinesVisible   = true;
extern string verticalLinesID        = "T3BurstLines";
extern color  verticalLinesUpColor   = LimeGreen;
extern color  verticalLinesDnColor   = Orange;
extern int    verticalLinesStyle     = STYLE_DOT;
extern int    verticalLinesWidth     = 0;

double MapBuffer[];
double upstate[];
double dnstate[];
double vel[];

//------------------------------------------------------------------
//
//------------------------------------------------------------------
int init()
{
    IndicatorBuffers(4);
    IndicatorDigits(6);
    SetIndexBuffer(0,MapBuffer);
    SetIndexBuffer(1,upstate);
    SetIndexBuffer(2,dnstate);
    SetIndexBuffer(3,vel);
      SetLevelValue(0,levelUp);
      SetLevelValue(1,levelDn);
    IndicatorShortName("T3 Velocity Burst "+T3_Period);
    return(0);
}
int deinit()
{
   string lookFor       = verticalLinesID+":";
   int    lookForLength = StringLen(lookFor);
   for (int i=ObjectsTotal()-1; i>=0; i--)
   {
      string objectName = ObjectName(i);
         if (StringSubstr(objectName,0,lookForLength) == lookFor) ObjectDelete(objectName);
   }
   return(0);
}


//------------------------------------------------------------------
//
//------------------------------------------------------------------
int start()
{
   int counted_bars = IndicatorCounted();
      if(counted_bars < 0) return(-1);
      if(counted_bars > 0) counted_bars--;
         int limit = MathMin(Bars-counted_bars,Bars-1);

   //
   //
   //
   //
   //
   
   for(int i=limit; i>=0; i--)
   {
      vel[i] = iVelocity(iMA(NULL,0,1,0,MODE_SMA,Price,i),Mom_Period,i,0);
      MapBuffer[i]=iT3(vel[i]-vel[i+1],T3_Period,b,T3_original,i);
      upstate[i] = upstate[i+1];
      dnstate[i] = dnstate[i+1];
         if (MapBuffer[i]>levelUp) upstate[i] =  1;
         if (MapBuffer[i]<levelUp) upstate[i] = -1;
         if (MapBuffer[i]>levelDn) dnstate[i] =  1;
         if (MapBuffer[i]<levelDn) dnstate[i] = -1;

         if (verticalLinesVisible)
         {
            ObjectDelete(verticalLinesID+":u:"+Time[i]);
            ObjectDelete(verticalLinesID+":d:"+Time[i]);
            if (upstate[i] != upstate[i+1] && upstate[i]== 1) drawLine(":u:",i,verticalLinesUpColor);
            if (dnstate[i] != dnstate[i+1] && dnstate[i]==-1) drawLine(":d:",i,verticalLinesDnColor);
         }            
   }      
   return(0);
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

void drawLine(string add, int i,color theColor)
{
   string name = verticalLinesID+add+Time[i];
   
      //
      //
      //
      //
      //
      
      ObjectCreate(name,OBJ_VLINE,0,Time[i],0);
         ObjectSet(name,OBJPROP_COLOR,theColor);
         ObjectSet(name,OBJPROP_STYLE,verticalLinesStyle);
         ObjectSet(name,OBJPROP_WIDTH,verticalLinesWidth);
         ObjectSet(name,OBJPROP_BACK,true);
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

double workT3[][6];
double workT3Coeffs[][6];
#define _period 0
#define _c1     1
#define _c2     2
#define _c3     3
#define _c4     4
#define _alpha  5

//
//
//
//
//

double iT3(double price, double period, double hot, bool original, int i, int instanceNo=0)
{
   if (ArrayRange(workT3,0) != Bars)                ArrayResize(workT3,Bars);
   if (ArrayRange(workT3Coeffs,0) < (instanceNo+1)) ArrayResize(workT3Coeffs,instanceNo+1);

   if (workT3Coeffs[instanceNo][_period] != period)
   {
     workT3Coeffs[instanceNo][_period] = period;
        double a = hot;
            workT3Coeffs[instanceNo][_c1] = -a*a*a;
            workT3Coeffs[instanceNo][_c2] = 3*a*a+3*a*a*a;
            workT3Coeffs[instanceNo][_c3] = -6*a*a-3*a-3*a*a*a;
            workT3Coeffs[instanceNo][_c4] = 1+3*a+a*a*a+3*a*a;
            if (original)
                 workT3Coeffs[instanceNo][_alpha] = 2.0/(1.0 + period);
            else workT3Coeffs[instanceNo][_alpha] = 2.0/(2.0 + (period-1.0)/2.0);
   }
   
   //
   //
   //
   //
   //
   
   int buffer = instanceNo*6;
   int r = Bars-i-1;
   if (r == 0)
      {
         workT3[r][0+buffer] = price;
         workT3[r][1+buffer] = price;
         workT3[r][2+buffer] = price;
         workT3[r][3+buffer] = price;
         workT3[r][4+buffer] = price;
         workT3[r][5+buffer] = price;
      }
   else
      {
         workT3[r][0+buffer] = workT3[r-1][0+buffer]+workT3Coeffs[instanceNo][_alpha]*(price              -workT3[r-1][0+buffer]);
         workT3[r][1+buffer] = workT3[r-1][1+buffer]+workT3Coeffs[instanceNo][_alpha]*(workT3[r][0+buffer]-workT3[r-1][1+buffer]);
         workT3[r][2+buffer] = workT3[r-1][2+buffer]+workT3Coeffs[instanceNo][_alpha]*(workT3[r][1+buffer]-workT3[r-1][2+buffer]);
         workT3[r][3+buffer] = workT3[r-1][3+buffer]+workT3Coeffs[instanceNo][_alpha]*(workT3[r][2+buffer]-workT3[r-1][3+buffer]);
         workT3[r][4+buffer] = workT3[r-1][4+buffer]+workT3Coeffs[instanceNo][_alpha]*(workT3[r][3+buffer]-workT3[r-1][4+buffer]);
         workT3[r][5+buffer] = workT3[r-1][5+buffer]+workT3Coeffs[instanceNo][_alpha]*(workT3[r][4+buffer]-workT3[r-1][5+buffer]);
      }

   //
   //
   //
   //
   //
   
   return(workT3Coeffs[instanceNo][_c1]*workT3[r][5+buffer] + 
          workT3Coeffs[instanceNo][_c2]*workT3[r][4+buffer] + 
          workT3Coeffs[instanceNo][_c3]*workT3[r][3+buffer] + 
          workT3Coeffs[instanceNo][_c4]*workT3[r][2+buffer]);
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

double prices[][2];
double iVelocity(double price, double length, int i, int instanceNo=0)
{
   if (ArrayRange(prices,0)!=Bars) ArrayResize(prices,Bars); i=Bars-i-1;
   
      prices[i][instanceNo] = price;
      double suma = 0.0, sumwa=0;
      double sumb = 0.0, sumwb=0;

      for(int k=0; k<length; k++)
      {
         double weight = length-k;
            suma  += prices[i-k][instanceNo] * weight;
            sumb  += prices[i-k][instanceNo] * weight * weight;
            sumwa += weight;
            sumwb += weight*weight;
      }
   return(sumb/sumwb-suma/sumwa);
}